@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2018 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,12 +36,7 @@ static void au_br_do_free(struct au_branch *br)
 	au_hnotify_fin_br(br);
 	/* always, regardless the mount option */
 	au_dr_hino_free(&br->br_dirren);
-
-	if (br->br_xino.xi_file)
-		fput(br->br_xino.xi_file);
-	for (i = br->br_xino.xi_nondir.total - 1; i >= 0; i--)
-		AuDebugOn(br->br_xino.xi_nondir.array[i]);
-	kfree(br->br_xino.xi_nondir.array);
+	au_xino_put(br);
 
 	AuDebugOn(au_br_count(br));
 	au_br_count_fin(br);
@@ -137,16 +133,12 @@ static struct au_branch *au_br_alloc(struct super_block *sb, int new_nbranch,
 	add_branch = kzalloc(sizeof(*add_branch), GFP_NOFS);
 	if (unlikely(!add_branch))
 		goto out;
-	add_branch->br_xino.xi_nondir.total = 8; /* initial size */
-	add_branch->br_xino.xi_nondir.array
-		= kcalloc(add_branch->br_xino.xi_nondir.total, sizeof(ino_t),
-			  GFP_NOFS);
-	if (unlikely(!add_branch->br_xino.xi_nondir.array))
+	add_branch->br_xino = au_xino_alloc(/*nfile*/1);
+	if (unlikely(!add_branch->br_xino))
 		goto out_br;
-
 	err = au_hnotify_init_br(add_branch, perm);
 	if (unlikely(err))
-		goto out_xinondir;
+		goto out_xino;
 
 	if (au_br_writable(perm)) {
 		/* may be freed separately at changing the branch permission */
@@ -178,8 +170,8 @@ out_wbr:
 	kfree(add_branch->br_wbr);
 out_hnotify:
 	au_hnotify_fin_br(add_branch);
-out_xinondir:
-	kfree(add_branch->br_xino.xi_nondir.array);
+out_xino:
+	au_xino_put(add_branch);
 out_br:
 	kfree(add_branch);
 out:
@@ -390,16 +382,15 @@ static int au_br_init(struct au_branch *br, struct super_block *sb,
 		      struct au_opt_add *add)
 {
 	int err;
+	struct au_branch *brbase;
+	struct file *xf;
 	struct inode *h_inode;
 
 	err = 0;
-	spin_lock_init(&br->br_xino.xi_nondir.spin);
-	init_waitqueue_head(&br->br_xino.xi_nondir.wqh);
 	br->br_perm = add->perm;
 	br->br_path = add->path; /* set first, path_get() later */
 	spin_lock_init(&br->br_dykey_lock);
 	au_br_count_init(br);
-	atomic_set(&br->br_xino_running, 0);
 	br->br_id = au_new_br_id(sb);
 	AuDebugOn(br->br_id < 0);
 
@@ -415,11 +406,13 @@ static int au_br_init(struct au_branch *br, struct super_block *sb,
 	}
 
 	if (au_opt_test(au_mntflags(sb), XINO)) {
+		brbase = au_sbr(sb, 0);
+		xf = au_xino_file(brbase->br_xino, /*idx*/-1);
+		AuDebugOn(!xf);
 		h_inode = d_inode(add->path.dentry);
-		err = au_xino_br(sb, br, h_inode->i_ino,
-				 au_sbr(sb, 0)->br_xino.xi_file, /*do_test*/1);
+		err = au_xino_init_br(sb, br, h_inode->i_ino, &xf->f_path);
 		if (unlikely(err)) {
-			AuDebugOn(br->br_xino.xi_file);
+			AuDebugOn(au_xino_file(br->br_xino, /*idx*/-1));
 			goto out_err;
 		}
 	}
@@ -509,6 +502,7 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 	struct dentry *root, *h_dentry;
 	struct inode *root_inode;
 	struct au_branch *add_branch;
+	struct file *xf;
 
 	root = sb->s_root;
 	root_inode = d_inode(root);
@@ -535,13 +529,10 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 	}
 
 	add_bindex = add->bindex;
-	if (!remount)
-		au_br_do_add(sb, add_branch, add_bindex);
-	else {
-		sysaufs_brs_del(sb, add_bindex);
-		au_br_do_add(sb, add_branch, add_bindex);
-		sysaufs_brs_add(sb, add_bindex);
-	}
+	sysaufs_brs_del(sb, add_bindex);	/* remove successors */
+	au_br_do_add(sb, add_branch, add_bindex);
+	sysaufs_brs_add(sb, add_bindex);	/* append successors */
+	dbgaufs_brs_add(sb, add_bindex, /*topdown*/0);	/* rename successors */
 
 	h_dentry = add->path.dentry;
 	if (!add_bindex) {
@@ -551,16 +542,17 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 		au_add_nlink(root_inode, d_inode(h_dentry));
 
 	/*
-	 * this test/set prevents aufs from handling unnecesary notify events
+	 * this test/set prevents aufs from handling unnecessary notify events
 	 * of xino files, in case of re-adding a writable branch which was
 	 * once detached from aufs.
 	 */
 	if (au_xino_brid(sb) < 0
 	    && au_br_writable(add_branch->br_perm)
-	    && !au_test_fs_bad_xino(h_dentry->d_sb)
-	    && add_branch->br_xino.xi_file
-	    && add_branch->br_xino.xi_file->f_path.dentry->d_parent == h_dentry)
-		au_xino_brid_set(sb, add_branch->br_id);
+	    && !au_test_fs_bad_xino(h_dentry->d_sb)) {
+		xf = au_xino_file(add_branch->br_xino, /*idx*/-1);
+		if (xf && xf->f_path.dentry->d_parent == h_dentry)
+			au_xino_brid_set(sb, add_branch->br_id);
+	}
 
 out:
 	return err;
@@ -863,7 +855,7 @@ out:
 }
 
 static void br_del_file(struct file **to_free, unsigned long long opened,
-			  aufs_bindex_t br_id)
+			aufs_bindex_t br_id)
 {
 	unsigned long long ull;
 	aufs_bindex_t bindex, btop, bbot, bfound;
@@ -1090,13 +1082,11 @@ int au_br_del(struct super_block *sb, struct au_opt_del *del, int remount)
 		di_write_lock_child(root);
 	}
 
-	if (!remount)
-		au_br_do_del(sb, bindex, br);
-	else {
-		sysaufs_brs_del(sb, bindex);
-		au_br_do_del(sb, bindex, br);
-		sysaufs_brs_add(sb, bindex);
-	}
+	sysaufs_brs_del(sb, bindex);	/* remove successors */
+	dbgaufs_xino_del(br);		/* remove one */
+	au_br_do_del(sb, bindex, br);
+	sysaufs_brs_add(sb, bindex);	/* append successors */
+	dbgaufs_brs_add(sb, bindex, /*topdown*/1);	/* rename successors */
 
 	if (!bindex) {
 		au_cpup_attr_all(d_inode(root), /*force*/1);
