@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2016 Junjiro R. Okajima
+ * Copyright (C) 2005-2018 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -181,7 +182,7 @@ out:
 	return err;
 }
 
-int au_xigen_set(struct super_block *sb, struct file *base)
+int au_xigen_set(struct super_block *sb, struct path *path)
 {
 	int err;
 	struct au_sbinfo *sbinfo;
@@ -190,7 +191,7 @@ int au_xigen_set(struct super_block *sb, struct file *base)
 	SiMustWriteLock(sb);
 
 	sbinfo = au_sbi(sb);
-	file = au_xino_create2(base, sbinfo->si_xigen);
+	file = au_xino_create2(sb, path, sbinfo->si_xigen);
 	err = PTR_ERR(file);
 	if (IS_ERR(file))
 		goto out;
@@ -200,6 +201,7 @@ int au_xigen_set(struct super_block *sb, struct file *base)
 	sbinfo->si_xigen = file;
 
 out:
+	AuTraceErr(err);
 	return err;
 }
 
@@ -416,7 +418,7 @@ static struct dentry *au_lkup_by_ino(struct path *path, ino_t ino,
 	}
 
 out_name:
-	au_delayed_free_page((unsigned long)arg.name);
+	free_page((unsigned long)arg.name);
 out_file:
 	fput(file);
 out:
@@ -570,7 +572,7 @@ out_relock:
 			dentry = ERR_PTR(-ESTALE);
 		}
 out_pathname:
-	au_delayed_free_page((unsigned long)pathname);
+	free_page((unsigned long)pathname);
 out_h_parent:
 	dput(h_parent);
 out:
