@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2018 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1442,7 +1443,8 @@ static int au_opt_simple(struct super_block *sb, struct au_opt *opt,
 
 	case Opt_trunc_xino_path:
 	case Opt_itrunc_xino:
-		err = au_xino_trunc(sb, opt->xino_itrunc.bindex);
+		err = au_xino_trunc(sb, opt->xino_itrunc.bindex,
+				    /*idx_begin*/0);
 		if (!err)
 			err = 1;
 		break;
@@ -1558,14 +1560,7 @@ static int au_opt_xino(struct super_block *sb, struct au_opt *opt,
 	err = 0;
 	switch (opt->type) {
 	case Opt_xino:
-		err = au_xino_set(sb, &opt->xino,
-				  !!au_ftest_opts(opts->flags, REMOUNT));
-		if (unlikely(err))
-			break;
-
-		*opt_xino = &opt->xino;
 		au_xino_brid_set(sb, -1);
-
 		/* safe d_parent access */
 		parent = opt->xino.file->f_path.dentry->d_parent;
 		root = sb->s_root;
@@ -1577,11 +1572,17 @@ static int au_opt_xino(struct super_block *sb, struct au_opt *opt,
 				break;
 			}
 		}
+
+		err = au_xino_set(sb, &opt->xino,
+				  !!au_ftest_opts(opts->flags, REMOUNT));
+		if (unlikely(err))
+			break;
+
+		*opt_xino = &opt->xino;
 		break;
 
 	case Opt_noxino:
 		au_xino_clr(sb);
-		au_xino_brid_set(sb, -1);
 		*opt_xino = (void *)-1;
 		break;
 	}
