@@ -1,5 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2018 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +36,7 @@ static struct inode *aufs_alloc_inode(struct super_block *sb __maybe_unused)
 	c = au_cache_alloc_icntnr();
 	if (c) {
 		au_icntnr_init(c);
-		c->vfs_inode.i_version = 1; /* sigen(sb); */
+		inode_set_iversion(&c->vfs_inode, 1); /* sigen(sb); */
 		c->iinfo.ii_hinode = NULL;
 		return &c->vfs_inode;
 	}
@@ -73,7 +74,7 @@ struct inode *au_iget_locked(struct super_block *sb, ino_t ino)
 	if (!err)
 		err = au_iinfo_init(inode);
 	if (!err)
-		inode->i_version++;
+		inode_inc_iversion(inode);
 	else {
 		iget_failed(inode);
 		inode = ERR_PTR(err);
@@ -245,6 +246,10 @@ static int aufs_show_options(struct seq_file *m, struct dentry *dentry)
 	sb = dentry->d_sb;
 	if (sb->s_flags & SB_POSIXACL)
 		seq_puts(m, ",acl");
+#if 0
+	if (sb->s_flags & SB_I_VERSION)
+		seq_puts(m, ",i_version");
+#endif
 
 	/* lock free root dinfo */
 	si_noflush_read_lock(sb);
@@ -466,11 +471,8 @@ static void aufs_put_super(struct super_block *sb)
 	struct au_sbinfo *sbinfo;
 
 	sbinfo = au_sbi(sb);
-	if (!sbinfo)
-		return;
-
-	dbgaufs_si_fin(sbinfo);
-	kobject_put(&sbinfo->si_kobj);
+	if (sbinfo)
+		kobject_put(&sbinfo->si_kobj);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -739,7 +741,7 @@ static void au_remount_refresh(struct super_block *sb, unsigned int do_idop)
 			AuDebugOn(sbi->si_iop_array == aufs_iop);
 			sbi->si_iop_array = aufs_iop;
 		}
-		pr_info("reset to %pf and %pf\n",
+		pr_info("reset to %ps and %ps\n",
 			sb->s_d_op, sbi->si_iop_array);
 	}
 
@@ -924,6 +926,7 @@ static int aufs_fill_super(struct super_block *sb, void *raw_data,
 
 	/* all timestamps always follow the ones on the branch */
 	sb->s_flags |= SB_NOATIME | SB_NODIRATIME;
+	sb->s_flags |= SB_I_VERSION; /* do we really need this? */
 	sb->s_op = &aufs_sop;
 	sb->s_d_op = &aufs_dop;
 	sb->s_magic = AUFS_SUPER_MAGIC;
@@ -958,7 +961,7 @@ static int aufs_fill_super(struct super_block *sb, void *raw_data,
 	au_opts_free(&opts);
 	if (!err && au_ftest_si(sbinfo, NO_DREVAL)) {
 		sb->s_d_op = &aufs_dop_noreval;
-		pr_info("%pf\n", sb->s_d_op);
+		pr_info("%ps\n", sb->s_d_op);
 		au_refresh_dop(root, /*force_reval*/0);
 		sbinfo->si_iop_array = aufs_iop_nogetattr;
 		au_refresh_iop(inode, /*force_getattr*/0);
@@ -972,7 +975,6 @@ out_root:
 	dput(root);
 	sb->s_root = NULL;
 out_info:
-	dbgaufs_si_fin(sbinfo);
 	kobject_put(&sbinfo->si_kobj);
 	sb->s_fs_info = NULL;
 out_opts:
@@ -991,7 +993,6 @@ static struct dentry *aufs_mount(struct file_system_type *fs_type, int flags,
 				 void *raw_data)
 {
 	struct dentry *root;
-	struct super_block *sb;
 
 	/* all timestamps always follow the ones on the branch */
 	/* mnt->mnt_flags |= MNT_NOATIME | MNT_NODIRATIME; */
@@ -999,11 +1000,7 @@ static struct dentry *aufs_mount(struct file_system_type *fs_type, int flags,
 	if (IS_ERR(root))
 		goto out;
 
-	sb = root->d_sb;
-	si_write_lock(sb, !AuLock_FLUSH);
-	sysaufs_brs_add(sb, 0);
-	si_write_unlock(sb);
-	au_sbilist_add(sb);
+	au_sbilist_add(root->d_sb);
 
 out:
 	return root;
